# Menu Migration

The Menu Migration module facilitates the import and export of menu hierarchies
from one Drupal copy to another one.

Out of the box, the module allows the following:
- Export menu hierarchies to a file in the codebase
- Import menu hierarchies from a file in the codebase
- Export menu hierarchies to a downloadable file
- Import menu hierarchies from an uploaded file

The module only supports **MenuLinkContent** menu items (the ones that are
manually created), any other menu items will not be exported/imported (e.g.
Views, Taonomy Menu etc.). It currently offers one format: JSON.

The export destinations, import sources and formats can be extended through the
Plugin system.

For a full description of the module, visit the
[project page](https://www.drupal.org/project/menu_migration).

For a detailed documentation, visit the
[module documentation](https://www.drupal.org/docs/extending-drupal/contributed-modules/contributed-module-documentation/menu-migration-import-export).

Submit bug reports and feature suggestions, or track changes in the
[issue queue](https://www.drupal.org/project/issues/menu_migration).

Chat, news, support & feedback in the dedicated [Slack channel](https://drupal.slack.com/archives/C079PP3DMJP).

## Table of contents

- [Requirements](#requirements)
- [Installation](#installation)
- [Configuration](#configuration)
- [Access and Permissions](#access-and-permissions)
- [Drush integration](#drush-integration)
- [Extending Destinations](#extending-destinations)
- [Extending Formats](#extending-formats)
- [Extending Sources](#extending-sources)
- [Troubleshooting](#troubleshooting)
- [Maintainers](#maintainers)

## Requirements

File permissions actions might be required on the server for the Codebase Export
Destination.

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

## Configuration

There are two configuration areas that are available:
1. Export Types - allows defining export types that can later trigger exports
2. Import Types - allows defining import types that can later trigger imports

### Defining an export type

In order to perform a menu export, an export type needs to be created.
1. Navigate to Configuration → Development → Menu Migration → Export Types
2. Click on _Add export type_ and fill out the form
3. The _Export type name_ can be anything relevant to you
4. The machine name will be the ID of the export type and will be used with
Drush
5. The _Export Destination_ can have one of the following values: Codebase or
Download
6. The Codebase exports the menus to the file system, and the Download generates
a downloadable file
7. Set the _Format_ to JSON (the only available option at the moment)
8. Select the _Menus_ that you wish to export
9. Fill out any other field depending on the _Export Destination_ choice.

### Export menu

1. Navigate to Configuration → Development → Menu Migration → Export Types
2. Here you have a listing of the available export types
3. In the _Operations_ column, click on _Export_ for one of the export types
4. You will be presented with a confirmation form that details all information
   about the export process & location
5. If you are satisfied with the conditions, click on export and that's it

### Defining an import type

In order to perform a menu import, an import type needs to be created.
1. Navigate to Configuration → Development → Menu Migration → Import Types
2. Click on _Add Import type_ and fill out the form
3. The _Import type name_ can be anything relevant to you
4. The machine name will be the ID of the Import type and will be used with
   Drush
5. The _Import Source_ can have one of the following values: Codebase or
   File Upload.
6. The Codebase Imports the menus from the file system, and the File Upload
   imports them from an uploaded file
7. Set the _Format_ to JSON (the only available option at the moment)
8. Select the _Menus_ that you wish to Import
9. Fill out any other field depending on the _Import Source_ choice.

### Import menu

1. Navigate to Configuration → Development → Menu Migration → Import Types
2. Here you have a listing of the available import types
3. In the _Operations_ column, click on _Import_ for one of the export types
4. You will be presented with a confirmation form that details all information
   about the import process & source
5. If you are satisfied with the conditions, click on export and that's it

## Access and Permissions

The following permissions are available:

| Machine name                               | Title                              | Usage                                                                                 |
|--------------------------------------------|------------------------------------|---------------------------------------------------------------------------------------|
| ` administer menu migration `              | Administer menu migration          | Bypasses any other menu_migration permission, and gives access to everything.         |
| ` administer menu migration export types ` | Administer export types            | Administer Export Types: add, edit, delete, export. Anything related to Export Types. |
| ` administer menu migration import types ` | Administer import types            | Administer Import Types: add, edit, delete, export. Anything related to Import Types. |
| ` perform export on menu migrations `      | Perform export on any Export Types | Access to the Export Types listing page and permission to perform exports.            |
| ` perform import on menu migrations `      | Perform import on any Import Types | Access to the Export Types listing page and permission to perform imports.            |

## Drush integration

Menu migration provides four Drush commands for managing imports and exports.
You can import one or more menus at a time through your already defined export
types and import types.

### Export Command

Performs the export for the given export type ID. Requires confirmation.

```shell
# Export the Export Type with ID my_export_id.
drush mme my_export_id
# Export the Export Type with ID my_export_id, and skip confirmation question.
drush mme my_export_id -y
```

### Export List Command

Lists all the available export types that support Drush.

```shell
# List all the available export entities
drush mmel
```

### Import Command

Performs the import for the given export type ID. Requires confirmation.

```shell
# Import the Import Type with ID my_import_id.
drush mmi my_import_id
# Import the Import Type with ID my_import_id, and skip the confirmation question.
drush mmi my_import_id -y
```

### Import List Command

Lists all the available import types that support Drush.

```shell
# List all the available import entities
drush mmil
```

## Extending Destinations

You can easily extend export destinations, and add your own custom destination
by implementing a MenuMigrationDestination plugin.

Existing examples with more code in:
- `src/Plugin/menu_migration/ExportDestination/Codebase.php`
- `src/Plugin/menu_migration/ExportDestination/Download.php`

Example:

File: `src/Plugin/menu_migration/ExportDestination/MyDestination.php`

File Contents:

```php
<?php

namespace Drupal\mymodule\Plugin\menu_migration\ExportDestination;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\menu_migration\Attribute\MenuMigrationDestination;
use Drupal\menu_migration\Plugin\menu_migration\ExportDestination\ExportDestinationBase;

/**
 * Provides a MyDestination export destination.
 */
#[MenuMigrationDestination(
  id: 'mydestination',
  label: new TranslatableMarkup('My Destination'),
  multiple: FALSE, # Optional, defaults to TRUE
  cli: TRUE # Optional, defaults to FALSE
)]
class MyDestination extends ExportDestinationBase {

  /**
   * {@inheritdoc}
   */
  public function exportMenu(string $menuName) {
    // TODO: Implement the export logic and return a boolean.
    // If the export fails, and you wish to display e detailed error message
    // about what went wrong, you interrupt the process and do the following:
    throw new MenuMigrationException(sprintf('My error message with the "%s" argument.', $argument));
  }

  /**
   * {@inheritdoc}
   */
  public function getExportDescription() {
    $description = parent::getExportDescription();
    // The parent contains the general export description. You can add details
    // specific to your export plugin here.
    $description[] = $this->t('The menus will have something special.');
    return $description;
  }

}
```

## Extending Formats

You can easily extend export/import formats, and add your own custom format by
implementing a MenuMigrationFormat plugin.

Existing examples with more code in:
- `src/Plugin/menu_migration/Format/JsonFormat.php`

Example:

File: `src/Plugin/menu_migration/Format/MyFormat.php`

File Contents:

```php
<?php

namespace Drupal\mymodule\Plugin\menu_migration\Format;

use Drupal\Component\Serialization\Json;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\menu_migration\Attribute\MenuMigrationFormat;
use Drupal\menu_migration\Plugin\menu_migration\Format\FormatBase;

/**
 * Provides the myFormat import/export Format.
 */
#[MenuMigrationFormat(
  id: 'myformat',
  label: new TranslatableMarkup('My Format')
)]
class MyFormat extends FormatBase {

  /**
   * {@inheritdoc}
   */
  public function encode(array $menuTree) {
    // Here you encode your menu tree. Example with JSON.
    return Json::encode($menuTree);
  }

  /**
   * {@inheritdoc}
   */
  public function decode(mixed $menuTree) {
    // Here you decode your menu tree. Example with JSON.
    return Json::decode($menuTree);
  }

  /**
   * {@inheritdoc}
   */
  public function allowedExtensions() {
    // Here you add allowed file extensions for sources like file_upload. If
    // applicable?
    return ['json'];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultExtension() {
    // Here you add the default file extension for exports. If applicable?
    return 'json';
  }

  /**
   * {@inheritdoc}
   */
  public function mimeType() {
    // Here you add the default file mimetype for exports like download. If
    // applicable?
    return 'application/json';
  }

}
```

## Extending Sources

You can easily extend import sources, and add your own custom sources by
implementing a MenuMigrationSource plugin.

Existing examples with more code in:
- `src/Plugin/menu_migration/ImportSource/Codebase.php`
- `src/Plugin/menu_migration/ImportSource/FileUpload.php`

Example:

File: `src/Plugin/menu_migration/ImportSource/MySource.php`

File Contents:

```php
<?php

namespace Drupal\mymodule\Plugin\menu_migration\ImportSource;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\menu_migration\Attribute\MenuMigrationSource;
use Drupal\menu_migration\Plugin\menu_migration\ImportSource\ImportSourceBase;

/**
 * Provides a MySource import source.
 */
#[MenuMigrationSource(
  id: 'mysource',
  label: new TranslatableMarkup('My Source'),
  multiple: FALSE, # Optional, defaults to TRUE
  cli: TRUE # Optional, defaults to FALSE
)]
class MySource extends ImportSourceBase {

  /**
   * {@inheritdoc}
   */
  public function importMenu(string $menuName) {
    // TODO: Implement the import logic and return a boolean.
    // If the import fails, and you wish to display e detailed error message
    // about what went wrong, you interrupt the process and do the following:
    throw new MenuMigrationException(sprintf('My error message with the "%s" argument.', $argument));
  }

  /**
   * {@inheritdoc}
   */
  public function getImportDescription() {
    $description = parent::getImportDescription();
    // The parent contains the general import description. You can add details
    // specific to your export plugin here.
    $description[] = $this->t('The menus will have something special.');
    return $description;
  }

}
```

## Troubleshooting

If you're getting errors that the menus can't be exported, please make
sure that the target directory has writing access by the server user that
is about to execute the exports when using the Codebase destination.

If the menu item hierarchy is messed up after import, make sure menu items are
only added via the menu UI (for example "Main navigation") and not a View.

## Maintainers

Cristina B. - [bbu23](https://www.drupal.org/u/bbu23)
