<?php

namespace Drupal\menu_migration\Commands;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\menu_migration\Entity\ExportTypeInterface;
use Drupal\menu_migration\Entity\ImportTypeInterface;
use Drupal\menu_migration\Service\MenuMigrationService;
use Drush\Commands\DrushCommands;

/**
 * Drush commands for the menu migration operations.
 */
class MenuMigrationCommands extends DrushCommands {

  /**
   * The menu migration service.
   *
   * @var \Drupal\menu_migration\Service\MenuMigrationService
   */
  protected MenuMigrationService $menuMigration;

  /**
   * The Export Type entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $exportTypeStorage;

  /**
   * The Import Type entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $importTypeStorage;

  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct();
    $this->exportTypeStorage = $entityTypeManager->getStorage('mm_export_type');
    $this->importTypeStorage = $entityTypeManager->getStorage('mm_import_type');
  }

  /**
   * Exports one or more menu items using the given export type ID.
   *
   * @param string $export_type
   *   The machine name (ID) of the Export Type.
   *
   * @command menu_migration:export
   *
   * @usage menu_migration:export my_export_type
   *   Export the configured menu(s) in the Export Type with my_export_type ID.
   *
   * @validate-module-enabled menu_migration
   *
   * @aliases mme
   */
  public function export(string $export_type = '') {
    if (empty($export_type)) {
      $this->io()->error(dt('You must specify the ID of the Export Type. Use drush mmel to see the available IDs.'));
    }
    else {
      $exportType = $this->exportTypeStorage->load($export_type);
      if ($exportType instanceof ExportTypeInterface) {
        if (!$exportType->handleCli()) {
          $this->io()->error(dt('The "!arg" Export Type does not support drush. Use drush mmel to see the supported IDs.', [
            '!arg' => $export_type,
          ]));
        }
        else {
          $exportDescription = $exportType->getDestinationPlugin()->getExportDescription();
          foreach ($exportDescription as $description) {
            $this->io()->writeln(strip_tags($description));
          }
          if ($this->io()->confirm('Are you sure?')) {
            $result = $exportType->export();
            if (!empty($result['success'])) {
              $this->io()->success(dt('The following menu(s) were successfully exported: !menus', [
                '!menus' => implode(', ', $result['success']),
              ]));
            }
            if (!empty($result['failure'])) {
              $this->io()->error(dt('The following menu(s) failed to be exported: !menus', [
                '!menus' => implode(', ', $result['failure']),
              ]));
            }
            if (!empty($result['errors'])) {
              // Display additional error messages if any.
              $this->io()->error($result['errors']);
            }
          }
        }
      }
      else {
        $this->io()->error(dt('The "!arg" Export Type does not exist. Please enter a valid Export Type.', ['!arg' => $export_type]));
      }
    }
  }

  /**
   * Lists the available export types.
   *
   * If there aren't any export types available, or none of the existing ones
   * are compatible with drush, an empty table is returned.
   *
   * @command menu_migration:export-list
   *
   * @usage menu_migration:export-list
   *   List all the available export types that are compatible with Drush.
   *
   * @validate-module-enabled menu_migration
   *
   * @aliases mmel
   */
  public function exportList() {
    /** @var \Drupal\menu_migration\Entity\ExportTypeInterface[] $exportTypes */
    $exportTypes = $this->exportTypeStorage->loadMultiple();
    $headers = [
      dt('ID'),
      dt('Label'),
      dt('Format'),
      dt('Destination'),
      dt('Menus'),
      dt('Command'),
    ];
    $types = [];
    foreach ($exportTypes as $exportType) {
      if ($exportType->handleCli()) {
        $destination = $exportType->getDestinationPlugin();
        $types[] = [
          $exportType->id(),
          $exportType->label(),
          $destination && $destination->getFormatPlugin() ? $destination->getFormatPlugin()->label() : '',
          $destination ? $destination->label() : '',
          implode(', ', $exportType->getMenus()),
          $exportType->getCommand(),
        ];
      }
    }
    $this->io()->table($headers, $types);
  }

  /**
   * Imports one or more menu items using the given import type ID.
   *
   * @param string $import_type
   *   The machine name (ID) of the Import Type.
   *
   * @command menu_migration:import
   *
   * @usage menu_migration:import my_import_type
   *   Import the configured menu(s) in the Import Type with my_import_type ID.
   *
   * @validate-module-enabled menu_migration
   *
   * @aliases mmi
   */
  public function import(string $import_type = '') {
    if (empty($import_type)) {
      $this->io()->error(dt('You must specify the ID of the Import Type. Use drush mmil to see the available IDs.'));
    }
    else {
      $importType = $this->importTypeStorage->load($import_type);
      if ($importType instanceof ImportTypeInterface) {
        if (!$importType->handleCli()) {
          $this->io()->error(dt('The "!arg" Import Type does not support drush. Use drush mmil to see the supported IDs.', [
            '!arg' => $import_type,
          ]));
        }
        else {
          $importDescription = $importType->getSourcePlugin()->getImportDescription();
          foreach ($importDescription as $description) {
            $this->io()->writeln(strip_tags($description));
          }
          if ($this->io()->confirm('Are you sure?')) {
            $result = $importType->import();
            if (!empty($result['success'])) {
              $this->io()->success(dt('The following menu(s) were successfully imported: !menus', [
                '!menus' => implode(', ', $result['success']),
              ]));
            }
            if (!empty($result['failure'])) {
              $this->io()->error(dt('The following menu(s) failed to be imported: !menus', [
                '!menus' => implode(', ', $result['failure']),
              ]));
            }
            if (!empty($result['errors'])) {
              // Display additional error messages if any.
              $this->io()->error($result['errors']);
            }
          }
        }
      }
      else {
        $this->io()->error(dt('The "!arg" Import Type does not exist. Please enter a valid Import Type.', ['!arg' => $import_type]));
      }
    }
  }

  /**
   * Lists the available import types.
   *
   * If there aren't any import types available, or none of the existing ones
   * are compatible with drush, an empty table is returned.
   *
   * @command menu_migration:import-list
   *
   * @usage menu_migration:import-list
   *   List all the available import types that are compatible with Drush.
   *
   * @validate-module-enabled menu_migration
   *
   * @aliases mmil
   */
  public function importList() {
    /** @var \Drupal\menu_migration\Entity\ImportTypeInterface[] $importTypes */
    $importTypes = $this->importTypeStorage->loadMultiple();
    $headers = [
      dt('ID'),
      dt('Label'),
      dt('Source'),
      dt('Format'),
      dt('Menus'),
      dt('Command'),
    ];
    $types = [];
    foreach ($importTypes as $importType) {
      if ($importType->handleCli()) {
        $source = $importType->getSourcePlugin();
        $types[] = [
          $importType->id(),
          $importType->label(),
          $source ? $source->label() : '',
          $source && $source->getFormatPlugin() ? $source->getFormatPlugin()->label() : '',
          implode(', ', $importType->getMenus()),
          $importType->getCommand(),
        ];
      }
    }
    $this->io()->table($headers, $types);
  }

}
